import CoreHaptics
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc

CHHapticCompletionHandler = b"v@"
CHHapticEngineFinishedHandler = objc._C_NSInteger + b"@"
CHHapticEngineStoppedHandler = b"v" + objc._C_NSInteger
CHHapticEngineResetHandler = b"v"


class TestCHHapticEngine(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(CoreHaptics.CHHapticEngineFinishedAction)
        self.assertIsEnumType(CoreHaptics.CHHapticEngineStoppedReason)

    def test_constants(self):
        self.assertEqual(CoreHaptics.CHHapticTimeImmediate, 0.0)

        self.assertEqual(CoreHaptics.CHHapticEngineFinishedActionStopEngine, 1)
        self.assertEqual(CoreHaptics.CHHapticEngineFinishedActionLeaveEngineRunning, 2)

        self.assertEqual(
            CoreHaptics.CHHapticEngineStoppedReasonAudioSessionInterrupt, 1
        )
        self.assertEqual(CoreHaptics.CHHapticEngineStoppedReasonApplicationSuspended, 2)
        self.assertEqual(CoreHaptics.CHHapticEngineStoppedReasonIdleTimeout, 3)
        self.assertEqual(CoreHaptics.CHHapticEngineStoppedReasonNotifyWhenFinished, 4)
        self.assertEqual(CoreHaptics.CHHapticEngineStoppedReasonEngineDestroyed, 5)
        self.assertEqual(
            CoreHaptics.CHHapticEngineStoppedReasonGameControllerDisconnect, 6
        )

        self.assertEqual(CoreHaptics.CHHapticEngineStoppedReasonSystemError, -1)

    @min_sdk_level("12.0")
    def test_constants12_0(self):
        self.assertIsInstance(
            CoreHaptics.CHHapticAudioResourceKeyUseVolumeEnvelope, str
        )

    @min_sdk_level("13.0")
    def test_constants13_0(self):
        self.assertIsInstance(CoreHaptics.CHHapticAudioResourceKeyLoopEnabled, str)

    @min_sdk_level("10.15")
    def test_methods10_15(self):
        self.assertResultIsBlock(
            CoreHaptics.CHHapticEngine.stoppedHandler, CHHapticEngineStoppedHandler
        )
        self.assertResultIsBlock(
            CoreHaptics.CHHapticEngine.resetHandler, CHHapticEngineResetHandler
        )

        self.assertArgIsBlock(
            CoreHaptics.CHHapticEngine.setStoppedHandler_,
            0,
            CHHapticEngineStoppedHandler,
        )
        self.assertArgIsBlock(
            CoreHaptics.CHHapticEngine.setResetHandler_, 0, CHHapticEngineResetHandler
        )

        self.assertResultIsBOOL(CoreHaptics.CHHapticEngine.playsHapticsOnly)
        self.assertResultIsBOOL(CoreHaptics.CHHapticEngine.isMutedForAudio)
        self.assertResultIsBOOL(CoreHaptics.CHHapticEngine.isMutedForHaptics)
        self.assertResultIsBOOL(CoreHaptics.CHHapticEngine.isAutoShutdownEnabled)

        self.assertArgIsBOOL(CoreHaptics.CHHapticEngine.setPlaysHapticsOnly_, 0)
        self.assertArgIsBOOL(CoreHaptics.CHHapticEngine.setIsMutedForAudio_, 0)
        self.assertArgIsBOOL(CoreHaptics.CHHapticEngine.setIsMutedForHaptics_, 0)
        self.assertArgIsBOOL(CoreHaptics.CHHapticEngine.setAutoShutdownEnabled_, 0)

        self.assertArgIsOut(CoreHaptics.CHHapticEngine.initAndReturnError_, 0)
        self.assertArgIsOut(CoreHaptics.CHHapticEngine.initWithAudioSession_error_, 1)

        self.assertArgIsBlock(
            CoreHaptics.CHHapticEngine.startWithCompletionHandler_,
            0,
            CHHapticCompletionHandler,
        )

        self.assertResultIsBOOL(CoreHaptics.CHHapticEngine.startAndReturnError_)
        self.assertArgIsOut(CoreHaptics.CHHapticEngine.startAndReturnError_, 0)

        self.assertArgIsBlock(
            CoreHaptics.CHHapticEngine.stopWithCompletionHandler_,
            0,
            CHHapticCompletionHandler,
        )

        self.assertArgIsBlock(
            CoreHaptics.CHHapticEngine.notifyWhenPlayersFinished_,
            0,
            CHHapticEngineFinishedHandler,
        )

        self.assertArgIsOut(
            CoreHaptics.CHHapticEngine.createPlayerWithPattern_error_, 1
        )
        self.assertArgIsOut(
            CoreHaptics.CHHapticEngine.createAdvancedPlayerWithPattern_error_, 1
        )
        self.assertArgIsOut(
            CoreHaptics.CHHapticEngine.registerAudioResource_options_error_, 2
        )

        self.assertResultIsBOOL(
            CoreHaptics.CHHapticEngine.unregisterAudioResource_error_, 1
        )
        self.assertArgIsOut(
            CoreHaptics.CHHapticEngine.unregisterAudioResource_error_, 1
        )

        self.assertResultIsBOOL(CoreHaptics.CHHapticEngine.playPatternFromURL_error_, 1)
        self.assertArgIsOut(CoreHaptics.CHHapticEngine.playPatternFromURL_error_, 1)

        self.assertResultIsBOOL(
            CoreHaptics.CHHapticEngine.playPatternFromData_error_, 1
        )
        self.assertArgIsOut(CoreHaptics.CHHapticEngine.playPatternFromData_error_, 1)

    @min_sdk_level("13.0")
    def test_methods13_0(self):
        self.assertResultIsBOOL(CoreHaptics.CHHapticEngine.playsAudioOnly)
        self.assertArgIsBOOL(CoreHaptics.CHHapticEngine.setPlaysAudioOnly_, 0)



import CoreHaptics
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc

CHHapticAdvancedPatternPlayerCompletionHandler = b"v@"


class TestCHHapticPatternPlayerHelper(CoreHaptics.NSObject):
    def startAtTime_error_(self, a, b):
        return 1

    def stopAtTime_error_(self, a, b):
        return 1

    def sendParameters_atTime_error_(self, a, b, c):
        return 1

    def scheduleParameterCurve_atTime_error_(self, a, b, c):
        return 1

    def cancelAndReturnError_(self, a):
        return 1

    def isMuted(self):
        return 1

    def setIsMuted_(self, a):
        return 1

    def pauseAtTime_error_(self, a, b):
        return 1

    def resumeAtTime_error_(self, a, b):
        return 1

    def seekToOffset_error_(self, a, b):
        return 1

    def loopEnabled(self):
        return 1

    def setLoopEnabled_(self, a):
        pass

    def loopEnd(self):
        return 1

    def setLoopEnd_(self, a):
        pass

    def playbackRate(self):
        return 1

    def setPlaybackRate_(self, a):
        pass

    # def isMuted(self): return 1
    # def setIsMuted_(self, a): pass
    def completionHandler(self):
        return 1

    def setCompletionHandler_(self, a):
        pass


class TestCHHapticPatternPlayer(TestCase):
    @min_sdk_level("10.15")
    def test_protocols(self):
        self.assertProtocolExists("CHHapticPatternPlayer")
        self.assertProtocolExists("CHHapticAdvancedPatternPlayer")

    def test_methods(self):
        # CHHapticPatternPlayer
        self.assertResultIsBOOL(TestCHHapticPatternPlayerHelper.startAtTime_error_)
        self.assertArgIsOut(TestCHHapticPatternPlayerHelper.startAtTime_error_, 1)

        self.assertResultIsBOOL(TestCHHapticPatternPlayerHelper.stopAtTime_error_)
        self.assertArgIsOut(TestCHHapticPatternPlayerHelper.stopAtTime_error_, 1)

        self.assertResultIsBOOL(
            TestCHHapticPatternPlayerHelper.sendParameters_atTime_error_
        )
        self.assertArgIsOut(
            TestCHHapticPatternPlayerHelper.sendParameters_atTime_error_, 2
        )

        self.assertResultIsBOOL(
            TestCHHapticPatternPlayerHelper.scheduleParameterCurve_atTime_error_
        )
        self.assertArgIsOut(
            TestCHHapticPatternPlayerHelper.scheduleParameterCurve_atTime_error_, 2
        )

        self.assertResultIsBOOL(TestCHHapticPatternPlayerHelper.cancelAndReturnError_)
        self.assertArgIsOut(TestCHHapticPatternPlayerHelper.cancelAndReturnError_, 0)

        self.assertResultIsBOOL(TestCHHapticPatternPlayerHelper.isMuted)
        self.assertArgIsBOOL(TestCHHapticPatternPlayerHelper.setIsMuted_, 0)

        # CHHapticAdvancedPatternPlayer
        self.assertResultIsBOOL(TestCHHapticPatternPlayerHelper.pauseAtTime_error_)
        self.assertArgHasType(
            TestCHHapticPatternPlayerHelper.pauseAtTime_error_, 0, objc._C_DBL
        )
        self.assertArgIsOut(TestCHHapticPatternPlayerHelper.pauseAtTime_error_, 1)

        self.assertResultIsBOOL(TestCHHapticPatternPlayerHelper.resumeAtTime_error_)
        self.assertArgHasType(
            TestCHHapticPatternPlayerHelper.resumeAtTime_error_, 0, objc._C_DBL
        )
        self.assertArgIsOut(TestCHHapticPatternPlayerHelper.resumeAtTime_error_, 1)

        self.assertResultIsBOOL(TestCHHapticPatternPlayerHelper.seekToOffset_error_)
        self.assertArgHasType(
            TestCHHapticPatternPlayerHelper.seekToOffset_error_, 0, objc._C_DBL
        )
        self.assertArgIsOut(TestCHHapticPatternPlayerHelper.seekToOffset_error_, 1)

        self.assertResultIsBOOL(TestCHHapticPatternPlayerHelper.loopEnabled)
        self.assertArgIsBOOL(TestCHHapticPatternPlayerHelper.setLoopEnabled_, 0)

        self.assertResultHasType(TestCHHapticPatternPlayerHelper.loopEnd, objc._C_DBL)
        self.assertArgHasType(
            TestCHHapticPatternPlayerHelper.setLoopEnd_, 0, objc._C_DBL
        )

        self.assertResultHasType(
            TestCHHapticPatternPlayerHelper.playbackRate, objc._C_FLT
        )
        self.assertArgHasType(
            TestCHHapticPatternPlayerHelper.setPlaybackRate_, 0, objc._C_FLT
        )

        self.assertResultIsBOOL(TestCHHapticPatternPlayerHelper.isMuted)
        self.assertArgIsBOOL(TestCHHapticPatternPlayerHelper.setIsMuted_, 0)

        self.assertResultIsBlock(
            TestCHHapticPatternPlayerHelper.completionHandler,
            CHHapticAdvancedPatternPlayerCompletionHandler,
        )
        self.assertArgIsBlock(
            TestCHHapticPatternPlayerHelper.setCompletionHandler_,
            0,
            CHHapticAdvancedPatternPlayerCompletionHandler,
        )



import CoreHaptics
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestCHHapticDeviceCapabilityHelper(CoreHaptics.NSObject):
    def minValue(self):
        return 1

    def maxValue(self):
        return 1

    def defaultValue(self):
        return 1

    def supportsHaptics(self):
        return 1

    def supportsAudio(self):
        return 1

    def attributesForEventParameter_eventType_error_(self, a, b, c):
        return 1

    def attributesForDynamicParameter_error_(self, a, b):
        return 1


class TestCHHapticDeviceCapability(TestCase):
    @min_sdk_level("10.15")
    def test_protocols(self):
        self.assertProtocolExists("CHHapticParameterAttributes")
        self.assertProtocolExists("CHHapticDeviceCapability")

    def test_methods(self):
        # CHHapticParameterAttributes
        self.assertResultHasType(
            TestCHHapticDeviceCapabilityHelper.minValue, objc._C_FLT
        )
        self.assertResultHasType(
            TestCHHapticDeviceCapabilityHelper.maxValue, objc._C_FLT
        )
        self.assertResultHasType(
            TestCHHapticDeviceCapabilityHelper.defaultValue, objc._C_FLT
        )

        # CHHapticDeviceCapability
        self.assertResultIsBOOL(TestCHHapticDeviceCapabilityHelper.supportsHaptics)
        self.assertResultIsBOOL(TestCHHapticDeviceCapabilityHelper.supportsAudio)

        self.assertArgHasType(
            TestCHHapticDeviceCapabilityHelper.attributesForEventParameter_eventType_error_,
            2,
            b"o^@",
        )
        self.assertArgHasType(
            TestCHHapticDeviceCapabilityHelper.attributesForDynamicParameter_error_,
            1,
            b"o^@",
        )



import CoreHaptics
from PyObjCTools.TestSupport import TestCase, min_sdk_level


class TestCHHapticParameter(TestCase):
    def test_typed_enum(self):
        self.assertIsTypedEnum(CoreHaptics.CHHapticEventParameterID, str)
        self.assertIsTypedEnum(CoreHaptics.CHHapticDynamicParameterID, str)

    @min_sdk_level("10.15")
    def test_constants10_15(self):
        self.assertIsInstance(CoreHaptics.CHHapticEventParameterIDHapticIntensity, str)
        self.assertIsInstance(CoreHaptics.CHHapticEventParameterIDHapticSharpness, str)
        self.assertIsInstance(CoreHaptics.CHHapticEventParameterIDAttackTime, str)
        self.assertIsInstance(CoreHaptics.CHHapticEventParameterIDDecayTime, str)
        self.assertIsInstance(CoreHaptics.CHHapticEventParameterIDReleaseTime, str)
        self.assertIsInstance(CoreHaptics.CHHapticEventParameterIDSustained, str)
        self.assertIsInstance(CoreHaptics.CHHapticEventParameterIDAudioVolume, str)
        self.assertIsInstance(CoreHaptics.CHHapticEventParameterIDAudioPitch, str)
        self.assertIsInstance(CoreHaptics.CHHapticEventParameterIDAudioPan, str)
        self.assertIsInstance(CoreHaptics.CHHapticEventParameterIDAudioBrightness, str)

        self.assertIsInstance(
            CoreHaptics.CHHapticDynamicParameterIDHapticIntensityControl, str
        )
        self.assertIsInstance(
            CoreHaptics.CHHapticDynamicParameterIDHapticSharpnessControl, str
        )
        self.assertIsInstance(
            CoreHaptics.CHHapticDynamicParameterIDHapticAttackTimeControl, str
        )
        self.assertIsInstance(
            CoreHaptics.CHHapticDynamicParameterIDHapticDecayTimeControl, str
        )
        self.assertIsInstance(
            CoreHaptics.CHHapticDynamicParameterIDHapticReleaseTimeControl, str
        )
        self.assertIsInstance(
            CoreHaptics.CHHapticDynamicParameterIDAudioVolumeControl, str
        )
        self.assertIsInstance(
            CoreHaptics.CHHapticDynamicParameterIDAudioPanControl, str
        )
        self.assertIsInstance(
            CoreHaptics.CHHapticDynamicParameterIDAudioBrightnessControl, str
        )
        self.assertIsInstance(
            CoreHaptics.CHHapticDynamicParameterIDAudioPitchControl, str
        )
        self.assertIsInstance(
            CoreHaptics.CHHapticDynamicParameterIDAudioAttackTimeControl, str
        )
        self.assertIsInstance(
            CoreHaptics.CHHapticDynamicParameterIDAudioDecayTimeControl, str
        )
        self.assertIsInstance(
            CoreHaptics.CHHapticDynamicParameterIDAudioReleaseTimeControl, str
        )
